@@ -1,10 +1,10 @@
 /***** BEGIN LICENSE BLOCK *****
- * Version: CPL 1.0/GPL 2.0/LGPL 2.1
+ * Version: EPL 1.0/GPL 2.0/LGPL 2.1
  *
- * The contents of this file are subject to the Common Public
+ * The contents of this file are subject to the Eclipse Public
  * License Version 1.0 (the "License"); you may not use this file
  * except in compliance with the License. You may obtain a copy of
- * the License at http://www.eclipse.org/legal/cpl-v10.html
+ * the License at http://www.eclipse.org/legal/epl-v10.html
  *
  * Software distributed under the License is distributed on an "AS
  * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
@@ -24,11 +24,11 @@
  * in which case the provisions of the GPL or the LGPL are applicable instead
  * of those above. If you wish to allow use of your version of this file only
  * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the CPL, indicate your
+ * use your version of this file under the terms of the EPL, indicate your
  * decision by deleting the provisions above and replace them with the notice
  * and other provisions required by the GPL or the LGPL. If you do not delete
  * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the CPL, the GPL or the LGPL.
+ * the terms of any one of the EPL, the GPL or the LGPL.
  ***** END LICENSE BLOCK *****/
 package org.jruby;
 
@@ -60,6 +60,7 @@ public class RubyNil extends RubyObject {
         RubyClass nilClass = runtime.defineClass("NilClass", runtime.getObject(), NIL_ALLOCATOR);
         runtime.setNilClass(nilClass);
         nilClass.index = ClassIndex.NIL;
+        nilClass.setReifiedClass(RubyNil.class);
         
         nilClass.defineAnnotatedMethods(RubyNil.class);
         
@@ -86,13 +87,18 @@ public class RubyNil extends RubyObject {
         return metaClass;
     }
     
+    @Override
+    public Class<?> getJavaClass() {
+        return void.class;
+    }
+    
     // Methods of the Nil Class (nil_*):
     
     /** nil_to_i
      *
      */
     @JRubyMethod(name = "to_i")
-    public static RubyFixnum to_i(IRubyObject recv) {
+    public static RubyFixnum to_i(ThreadContext context, IRubyObject recv) {
         return RubyFixnum.zero(recv.getRuntime());
     }
     
@@ -101,24 +107,29 @@ public class RubyNil extends RubyObject {
      *
      */
     @JRubyMethod(name = "to_f")
-    public static RubyFloat to_f(IRubyObject recv) {
-        return RubyFloat.newFloat(recv.getRuntime(), 0.0D);
+    public static RubyFloat to_f(ThreadContext context, IRubyObject recv) {
+        return RubyFloat.newFloat(context.runtime, 0.0D);
     }
     
     /** nil_to_s
      *
      */
     @JRubyMethod(name = "to_s")
-    public static RubyString to_s(IRubyObject recv) {
-        return RubyString.newEmptyString(recv.getRuntime());
+    public static RubyString to_s(ThreadContext context, IRubyObject recv) {
+        return RubyString.newEmptyString(context.runtime);
     }
     
     /** nil_to_a
      *
      */
     @JRubyMethod(name = "to_a")
-    public static RubyArray to_a(IRubyObject recv) {
-        return recv.getRuntime().newEmptyArray();
+    public static RubyArray to_a(ThreadContext context, IRubyObject recv) {
+        return context.runtime.newEmptyArray();
+    }
+    
+    @JRubyMethod(name = "to_h")
+    public static RubyHash to_h(ThreadContext context, IRubyObject recv) {
+        return RubyHash.newSmallHash(context.runtime);
     }
     
     /** nil_inspect
@@ -126,13 +137,18 @@ public class RubyNil extends RubyObject {
      */
     @JRubyMethod(name = "inspect")
     public static RubyString inspect(IRubyObject recv) {
-        return recv.getRuntime().newString("nil");
+        Ruby runtime = recv.getRuntime();
+        if (runtime.is1_9()) {
+            return RubyString.newUSASCIIString(runtime, "nil");
+        } else {
+            return RubyString.newString(runtime, "nil");
+        }
     }
     
     /** nil_type
      *
      */
-    @JRubyMethod(name = "type")
+    @JRubyMethod(name = "type", compat = CompatVersion.RUBY1_8)
     public static RubyClass type(IRubyObject recv) {
         return recv.getRuntime().getNilClass();
     }
@@ -176,8 +192,51 @@ public class RubyNil extends RubyObject {
         return this;
     }
 
+    /** nilclass_to_c
+     * 
+     */
+    @JRubyMethod(name = "to_c", compat = CompatVersion.RUBY1_9)
+    public static IRubyObject to_c(ThreadContext context, IRubyObject recv) {
+        return RubyComplex.newComplexCanonicalize(context, RubyFixnum.zero(context.runtime));
+    }
+    
+    /** nilclass_to_r
+     * 
+     */
+    @JRubyMethod(name = "to_r", compat = CompatVersion.RUBY1_9)
+    public static IRubyObject to_r(ThreadContext context, IRubyObject recv) {
+        return RubyRational.newRationalCanonicalize(context, RubyFixnum.zero(context.runtime));
+    }
+
+    /** nilclass_rationalize
+     *
+     */
+    @JRubyMethod(name = "rationalize", optional = 1, compat = CompatVersion.RUBY1_9)
+    public static IRubyObject rationalize(ThreadContext context, IRubyObject recv, IRubyObject[] args) {
+        return to_r(context, recv);
+    }
+
     @Override
-    public IRubyObject freeze(ThreadContext context) {
-        return this;
+    public Object toJava(Class target) {
+        if (target.isPrimitive()) {
+            if (target == Boolean.TYPE) {
+                return Boolean.FALSE;
+            } else if (target == Byte.TYPE) {
+                return (byte)0;
+            } else if (target == Short.TYPE) {
+                return (short)0;
+            } else if (target == Character.TYPE) {
+                return (char)0;
+            } else if (target == Integer.TYPE) {
+                return 0;
+            } else if (target == Long.TYPE) {
+                return (long)0;
+            } else if (target == Float.TYPE) {
+                return (float)0;
+            } else if (target == Double.TYPE) {
+                return (double)0;
+            }
+        }
+        return null;
     }
 }
