@@ -1,5 +1,5 @@
 /***** BEGIN LICENSE BLOCK *****
- * Version: CPL 1.0/GPL 2.0/LGPL 2.1
+ * Version: EPL 1.0/GPL 2.0/LGPL 2.1
  *
  * The contents of this file are subject to the Common Public
  * License Version 1.0 (the "License"); you may not use this file
@@ -25,26 +25,27 @@
  * in which case the provisions of the GPL or the LGPL are applicable instead
  * of those above. If you wish to allow use of your version of this file only
  * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the CPL, indicate your
+ * use your version of this file under the terms of the EPL, indicate your
  * decision by deleting the provisions above and replace them with the notice
  * and other provisions required by the GPL or the LGPL. If you do not delete
  * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the CPL, the GPL or the LGPL.
+ * the terms of any one of the EPL, the GPL or the LGPL.
  ***** END LICENSE BLOCK *****/
 package org.jruby.test;
 
 import junit.framework.Test;
 import junit.framework.TestSuite;
 
-import org.jruby.ast.executable.YARVMachineTest;
-import org.jruby.ast.visitor.rewriter.VisitorTestSuite;
 import org.jruby.ext.posix.JavaFileStatTest;
+import org.jruby.javasupport.TestJava;
 import org.jruby.javasupport.TestJavaClass;
 import org.jruby.javasupport.test.JavaSupportTestSuite;
-import org.jruby.runtime.callback.TestReflectionCallback;
+import org.jruby.runtime.EventHookTest;
 import org.jruby.util.JRubyThreadContextTest;
-import org.jruby.util.PlatformTest;
 import org.jruby.util.ShellLauncherTest;
+import org.jruby.util.TimeOutputFormatterTest;
+import org.jruby.util.log.ParameterizedWriterTest;
+import org.jruby.util.log.StandardErrorLoggerTest;
 
 /**
  *
@@ -62,6 +63,7 @@ public class MainTestSuite extends TestSuite {
         suite.addTestSuite(TestRuby.class);
         suite.addTestSuite(TestRequire.class);
         suite.addTestSuite(TestJavaUtil.class);
+        suite.addTestSuite(TestJava.class);
         suite.addTestSuite(TestJavaClass.class);
         suite.addTestSuite(TestKernel.class);
         suite.addTestSuite(TestRubyCollect.class);
@@ -70,22 +72,28 @@ public class MainTestSuite extends TestSuite {
         suite.addTest(JavaSupportTestSuite.suite());
         suite.addTestSuite(TestCommandlineParser.class);
         suite.addTestSuite(TestRubyException.class);
-        suite.addTestSuite(TestReflectionCallback.class);
         suite.addTestSuite(TestAdoptedThreading.class);
         suite.addTestSuite(TestRubyArray.class);
-        suite.addTestSuite(TestVariableCreation.class);
-        suite.addTestSuite(YARVMachineTest.class);
         suite.addTestSuite(TestRaiseException.class);
-        suite.addTest(VisitorTestSuite.suite());
-        suite.addTestSuite(PlatformTest.class);
         suite.addTestSuite(ShellLauncherTest.class);
         suite.addTestSuite(TestRbConfigLibrary.class);
-        suite.addTestSuite(TestParser.class);    
+        suite.addTestSuite(TestParser.class);
         suite.addTestSuite(TestRubyBigDecimal.class);
         suite.addTestSuite(JRubyThreadContextTest.class);
         suite.addTestSuite(JavaFileStatTest.class);
         suite.addTestSuite(TestCodeCache.class);
         suite.addTestSuite(TestJavaReentrantExceptions.class);
+        suite.addTestSuite(EventHookTest.class);
+        suite.addTestSuite(TestMethodFactories.class);
+        suite.addTestSuite(TimeOutputFormatterTest.class);
+        suite.addTestSuite(org.jruby.lexer.yacc.ByteArrayLexerSourceTest.class);
+        suite.addTestSuite(org.jruby.runtime.load.LoadServiceResourceInputStreamTest.class);
+        suite.addTestSuite(TestRubyString.class);
+        suite.addTestSuite(TestRubyNKF.class);
+        suite.addTestSuite(StandardErrorLoggerTest.class);
+        suite.addTestSuite(ParameterizedWriterTest.class);
+        suite.addTestSuite(TestRubyRational.class);
+        suite.addTestSuite(TestRecursiveCheck.class);
         return suite;
     }
 }
